from .base_generator import BaseGenerator

class ListGenerator(BaseGenerator):
    def __init__(self, input_data: list[str], generator, output_dir: str, progress_callback=None):
        super().__init__(generator, output_dir,progress_callback)
        self.data_list = input_data

    def _generate(self):
        total = len(self.data_list)

        for idx, data in enumerate(self.data_list, start=1):
            self._save_qr(data)

            if self.progress_callback:
                percent = (idx / total) * 100
                callback_data = {
                    "current_data": data,
                    "percent_complete": percent
                }
                self.progress_callback(callback_data)





from abc import ABC, abstractmethod
from pathlib import Path

class BaseGenerator(ABC):
    def __init__(self, generator, output_dir: str|Path, progress_callback=None):
        self.generator = generator
        self.output_dir:Path = Path(output_dir)
        self.progress_callback = progress_callback

    def generate(self):
        self.output_dir.mkdir(parents=True, exist_ok=True)
        self._generate()

    def _save_qr(self, data: str):
        safe_name = data.replace('/', '_').replace('\\', '_')
        file_path = self.output_dir / f"{safe_name}.png"
        self.generator.generate_qr(data, output_file_path=str(file_path))

    @abstractmethod
    def _generate(self):
        pass


import qrcode
from .qr_config import QrConfig

class QRGenerator:
    def __init__(self, config: QrConfig):
        self.config = config

    def generate_qr(self, data:str, output_file_path:str):
        qr = qrcode.QRCode(
            version=self.config.version,
            error_correction=self.config.error_correction,
            box_size=self.config.box_size,
            border=self.config.border,
        )
        qr.add_data(data)
        qr.make(fit=True)
        img = qr.make_image(fill_color=self.config.fill_color, back_color=self.config.back_color)
        img.save(output_file_path)


from .qr.qr_generator import QRGenerator
from .qr.qr_config import QrConfig
from .generators.single_generator import SingleGenerator
from .generators.list_generator import ListGenerator
from .generators.range_generator import RangeGenerator

__all__ = [
    "QRGenerator",
    "QrConfig",
    "SingleGenerator",
    "ListGenerator",
    "RangeGenerator",
]


import qrcode

class QrConfig:
    def __init__(self, box_size=10, border=1, fill_color="black", back_color="white"):
        self.version = 1
        self.error_correction = qrcode.constants.ERROR_CORRECT_H
        self.box_size = box_size
        self.border = border
        self.fill_color = fill_color
        self.back_color = back_color


from .base_generator import BaseGenerator
from pathlib import Path

class SingleGenerator(BaseGenerator):
    def __init__(self, input_data: str, generator, output_dir: str | Path,progress_callback=None):
        super().__init__(generator, output_dir,progress_callback)
        self.data = input_data

    def _generate(self):
        self._save_qr(self.data)
        if self.progress_callback:
            callback_data = {
                'current_data': self.data,
                'percent_complete': 100.0
            }
            self.progress_callback(callback_data)


from .base_generator import BaseGenerator

from pathlib import Path

class RangeGenerator(BaseGenerator):
    def __init__(self, input_data: dict, generator, output_dir: str | Path, progress_callback=None):
        super().__init__(generator,output_dir,progress_callback)
        self.prefix = input_data.get('prefix','')
        self.suffix = input_data.get('suffix','')
        self.start = input_data['start']
        self.end = input_data['end']


    def _generate(self):
        for i in range(self.start, self.end+1):
            data = self.prefix + str(i) + self.suffix
            self._save_qr(data)
            if self.progress_callback:
                callback_data = {
                    'current_data': data,
                    'percent_complete': ((i - self.start + 1) / (self.end - self.start + 1)) * 100
                }
                self.progress_callback(callback_data)

